function  [G1, impact, eu] = solveTeoreticalModel(spar, rpar, nvar, i);
% project: infor IV
% Given structural parameters and parameters of reduced form equations
% constructs and solves theoretical model using Sim's Gensis program
% =========================================================================
% Input: 
% spar   -  vector of structural parameters
% rpar   -  matrix of parameters from reduced form model, companion form
% i      -  indicator, 1 - sticky price model, 2 - sticky price model with 
%           indexation, 3 - sticky information model
% nvar   -  number of variables in VAR
% =========================================================================
% Output:  
% Solution in the following form:
% x_t =G1*x_(t-1) + impact*e_t
% G1, impact
% eu - vector (1,1) solution exists and it is unique
% =========================================================================

% Step 1. calibrated parameters:
b   = spar(1);   % beta, consumer discount factor
la  = spar(2);   % lambda, only for sticky price or sticky price 
                % with indexation (nonlinear combination of stickiness, 
                % and other structural parameters)
th  = spar(3);   % information stickiness parameter (only for sticky 
                % information model) 
xi  = spar(4);   % nonlinear combination of deep structural parameters
                % only for sticky information model

                
% Step 2. Rewriting equations in form of theoretical model                
%  Reduced form equations, in companion form                 
%  order in VAR is Z[t] = (pi[t-1], yg[t-1], dy[t-1], r[t-1],...)
n = size(rpar,2);

if i == 1;
    %  Sticky Price Phillips Curve 
    % eq(1:n-1) tZ[t] = rpar tZ[t-1] + tw[t] %tZ - tilde Z 
    g0 = eye(n-1); g0 = [zeros(n-1,1) g0; zeros(1,n)];
    g0 = [g0 zeros(n,1); zeros(1,n+1)];% to accomodate error term
    g1 = [rpar zeros(n-1,1);zeros(2,n+1)];

    % eq(n):  b*E[t-1]*pi[t]=pi[t-1]-la*yg[t-1]- v[t-1];
    g0(n,1) = b; % coefficient on current inflation
    g1(n,1) = 1; % coefficient on previous period inflation
    g1(n,2) = -la; % coefficient on previous period output
    g1(n,n+1) = -1; % coefficient on error term
    % eq(n+1): v[t] = v[t] 
    g0(n+1,n+1) = 1;
    
    c =zeros(size(g0,1),1); % constants  
    pi=[zeros(n-1,1);1;0]; % equations with expectations operators 
    psi=[[zeros(n,1);1] [eye(nvar-1); zeros(n-nvar+2,nvar-1)]]; % equations 
    % with errors (order of shocks is v[t] eps_yg[t], eps_dy[t], eps_r[t])
        
elseif i == 2; 
    %  Sticky Price Phillips Curve 
    if nvar ==n;
        % eq(1:n-1): tZ[t] = rpar tZ[t-1] + tw[t] %tZ - tilde Z
        g0 = eye(n-1); g0 = [zeros(n-1,1) g0 zeros(n-1,2); zeros(2,n+2)];
        g1 = [rpar zeros(n-1,2);zeros(3,n+2)];
        % eq(n): b/(1+b)*E[t-1]*pi[t]=pi[t-1] - 1/(1+b)*pi[1,t-1]-la/(1+b)*yg[t-1]- v[t-1];
        g0(n,1) = b/(1+b); % coefficient on current inflation
        g1(n,1) = 1; % coefficient on previous period inflation
        g1(n,2) = -la/(1+b); % coefficient on previous period output
        g1(n,n+1) = -1/(1+b); % coefficient on inflation two periods ago
        g1(n,n+2) = -1; % coefficient on previous period supply shock 
        % eq(n+1):  pi[1,t] = pi[t-1]
        g0(n+1,n+1) = 1; % coefficient on pi[1,t]
        g1(n+1,1) = 1; 
        % eq(n+2):  v[t] = v[t];
        g0(n+2,n+2) = 1;
        
        c =zeros(size(g0,1),1); % constants  
        pi=[zeros(n-1,1);1; 0; 0]; % equations with expectations operators 
        psi=[[zeros(n+1,1);1] [eye(nvar-1); zeros(3,nvar-1)]]; % equations 
        % with errors (order of shocks is v[t] eps_yg[t], eps_dy[t], eps_r[t])
    else; 
        % eq(1:n-1): tZ[t] = rpar tZ[t-1] + tw[t] %tZ - tilde Z
        g0 = eye(n-1); g0 = [zeros(n-1,1) g0 zeros(n-1,1); zeros(2,n+1)];
        g1 = [rpar zeros(n-1,1);zeros(2,n+1)];
        % eq(n): b/(1+b)*E[t-1]*pi[t]=pi[t-1] - 1/(1+b)*pi[1,t-1]-la/(1+b)*yg[t-1]- v[t-1];
        g0(n,1) = b/(1+b); % coefficient on current inflation
        g1(n,1) = 1; % coefficient on previous period inflation
        g1(n,2) = -la/(1+b); % coefficient on previous period output
        g1(n,nvar+1) = -1/(1+b); % coefficient on inflation two periods ago
        g1(n,n+1) = -1; % coefficient on previous period supply shock 
        % eq(n+1):  v[t] = v[t];
        g0(n+1,n+1) = 1;
        
        c =zeros(size(g0,1),1); % constants  
        pi=[zeros(n-1,1);1; 0]; % equations with expectations operators 
        psi=[[zeros(n,1);1] [eye(nvar-1); zeros(n-(nvar-2),nvar-1)]]; % equations 
        % with errors (order of shocks is v[t] eps_yg[t], eps_dy[t],
        % eps_r[t])
    end;
else; 
    %  Sticky information Phillips Curve
    th_i = th;
    ka_i = xi*(1-th_i)/th_i;
    % eq(1:n-1): tZ[t] = rpar tZ[t-1] + tw[t] %tZ - tilde Z 
    g00 = eye(n-1); g00 = [zeros(n-1,1) g00 zeros(n-1,1); zeros(1,n+1)];
    g11 = [rpar zeros(n-1,1);zeros(1,n+1)];
    
    %  introducing additional variable (inside of expectation operator)
    % eq(n): e[t]-pi[t]-xi*yg[t] = -xi*yg[t-1];
    g00(n,n+1) =1; g00(n,1) = -1; g00(n,2) = -xi; g11(n,2) = -xi; 
    
    
    % adjustable part of the system:
    % additional varialbes : ex e1[t] = (e[0,t],e[0,1,t],e[1,t],..., e[3,3,t], e[3,2,t], e[3,1,t],e[3,t])
    m = size(g00,1); n = size(g00,2);
    i_max = 5; 
    jj = (1:1:i_max+1)';
    N  = sum(jj);
    g0 = zeros(m+1+N,m+1+N); g1 = g0;
    g0(1:m,1:n) = g00; g1(1:m,1:n) = g11;

    % g0
    % eq(n+2): pi[t]- ka_i*yg[t]-v[t] = (1-th_i)*e[0,t-1]; 
     g0(m+1,1) = 1; g0(m+1,2) = -ka_i;
     g0(m+2:m+1+N,n:n+N-1) = eye(N);
     g1(m+1,n+1) = (1-th_i); 
    % e[0,t-1] = E[t-1]*e[t];     
    g1(n+1,n+1) = 1; % e5
    % g1    
    k  = n+1;
    % pi:
        I = eye(i_max+2);
        pi = zeros(size(g0,1),i_max+2);
        pi(1,:) = I(1,:);
        pi(k,:) = I(2,:);
    for ii = 1:size(jj)-1;
        k = k+ii;
        g1(m+1,k) = th_i^(ii)*(1-th_i);
        mat = zeros(ii+1,ii+1); mat(1,end) = 1;
        mat(2:end,2:end) = eye(ii);
        g1(k:k+ii,k:k+ii)=mat;
        % pi:
        pi(k,:) = I(ii+2,:);
    end
    pi = pi(:,2:end);
    c =zeros(size(g0,1),1);
    psi=[[zeros(nvar,1);1] [eye(nvar-1); zeros(2,nvar-1)];...
        zeros(size(pi,1)-(nvar+1),nvar)]; % equations 
        % with errors (order of shocks is v[t] eps_yg[t], eps_dy[t],
        % eps_r[t])
end;

[G1,C,impact,fmat,fwt,ywt,gev,eu]=gensys(g0,g1,c,psi,pi);
